program _rdqtese, eclass
	version 10
	*Back-office program for RD quantile treatment effects. This program does the nuts and bolts
	*_rdqteu is the same as _rdqte except it uses the uniform kernel
	* version se calculates analytical standard errors
	syntax varlist(min=2 max=2 numeric) [if] [in]  , RUNNING(varname) QUANTILES(numlist) HMEANS(numlist)  
	marksample touse
	
	tempname LQTE FC1inv FC0inv y0 y1 lo hi h2 h11 h10 h00 h01 q dplus dminus lqte ///
	lqtec ub lb hy hmean_1 hmean_0 h_half_1 h_half_0 hmean_11 hmean_10 hmean_01 hmean_00 ///
	h_half_11 h_half_10 h_half_01 h_half_00 h2_11 h2_10 h2_01 h2_00 sdr h2factor_00 ///
	h2factor_01 h2factor_10 h2factor_11 conv0 conv1 qy fy fyalt hd i11 i10 i01 i00 hy11 hy10 hy01 hy00 rbw fr se
	tempvar z phimean_1 phimean_0 phimean_11 phimean_10 phimean_01 phimean_00 phi_11 ///
	phi_10 phi_01 phi_00 ky phid_1 phid_0 f0 f1 f00 f11 f10 f01 kr

quietly {

preserve
keep if `touse'
tokenize `varlist'
local y "`1'"
local d "`2'"
local r "`running'"
gen `z'=`r'>=0

/* We have 6 different sets of bandwidths (and weights) to generate. A negative bandwidth means no data in the 
relevant cell (e.g., a sharp or partially fuzzy design). The hmeans macro has the following
 six bandwidths for means, in order*/
*1. hd_1=for estimating E[D|r=0] from above
*2. h_11= for Z=1,D=1
*3. h_10= for Z=1,D=0
*4. hd_0=for estimating E[D|r=0] from below
*5. h_01= for Z=0,D=1
*6. h_00= for Z=0,D=0

local hd_1: word 1 of `hmeans'
local hm_11: word 2 of `hmeans'
local hm_10: word 3 of `hmeans'
local hd_0: word 4 of `hmeans'
local hm_01: word 5 of `hmeans'
local hm_00: word 6 of `hmeans'

* Compute some reference bandwidths from the given mean bandwidths, and initialize weight variables:
foreach Zval in 1 0 {
    gen `phid_`Zval''=.5*(abs( `r'/ `hd_`Zval'')<=1)/ `hd_`Zval''
    foreach Dval in 1 0 {
		if `hm_`Zval'`Dval''>0 {
			scalar `i`Zval'`Dval''=1
			scalar `hmean_`Zval'`Dval''=`hm_`Zval'`Dval''		
			scalar `h_half_`Zval'`Dval''=`hmean_`Zval'`Dval''*(.5*(1-.5)/(normalden(invnormal(.5)))^2)^(1/5)
	
			gen `phimean_`Zval'`Dval''=.5*(abs( `r'/ `hmean_`Zval'`Dval'')<=1)/ `hmean_`Zval'`Dval''
			gen `phi_`Zval'`Dval''=1
			*calc rescaling factor for h2 bw as ratio of SDs of r and y
			sum `r' if `z'==`Zval' & `d' ==`Dval'
			scalar `sdr'=r(sd)
			sum `y' if `z'==`Zval' & `d' ==`Dval'
			scalar `h2factor_`Zval'`Dval''=r(sd)/`sdr'
			sum `y' if `z'==`Zval' & `d' ==`Dval'
			scalar `hy`Zval'`Dval''=1.06*r(sd)*r(N)^(-1/5)
		}
		else {
			scalar `i`Zval'`Dval''=0
			scalar `h2_`Zval'`Dval''=-1
			gen `phi_`Zval'`Dval''=.			
		}
    }
}

* kernel weights for mean in R-direction:
	* Calculate dplus = E[D|R=0+] and dminus = E[D|R=0-] using hd
if `i01'==1 {
	reg `d' `r' [aw=`phid_0'] if `z'==0
	scalar `dminus'=_b[_cons]
}
else scalar `dminus'=0
if `i10'==1{
	reg `d' `r' [aw=`phid_1'] if `z'==1
	scalar `dplus'=_b[_cons]
}
else scalar `dplus'=1
display "E[D|R=0-] = " `dminus'
display "E[D|R=0+] = " `dplus'
	
local qlength=0	
foreach quant of numlist `quantiles' {
	local ++qlength
}

*Initialize results matrices
matrix `lqte'=J(`qlength',1,0)
matrix `lb'=`lqte'
matrix `ub'=`lqte'
matrix `se'=`lqte'
gen `kr'=.

local qelem=0	
foreach quant of numlist `quantiles' {
	quietly {
	local ++qelem
	scalar `q'=`quant'
	local cent = 100 * `quant'
	foreach Zval in 1 0 {
    		foreach Dval in 1 0 {
			if `i`Zval'`Dval''==1 {
				*Conditional density estimation for plugin bw selection.
				*Guess the "y" for the point of density estimation as the q-th quantile
				*of y conditional on z=zval, d=dval, restricting attention to 3 bws from zero.
				
				centile `y' if `z'==`Zval' & `d' ==`Dval' & abs(`r')<=3*`hmean_`Zval'`Dval'', centile(`cent')
				if r(N)==0 centile `y' if `z'==`Zval' & `d' ==`Dval', centile(`cent')
				scalar `qy'=r(c_1)
				*Now do local linear density estimation:
				gen `ky' =1/`hy`Zval'`Dval''*normalden((`y'-`qy')/`hy`Zval'`Dval'')
				sum `ky' if `z'==`Zval' & `d' ==`Dval'
				sum `r'  if `z'==`Zval' & `d' ==`Dval'
				reg `ky' `r' [aw=`phimean_`Zval'`Dval''] if `z'==`Zval' & `d' ==`Dval'
				scalar `fy'=_b[_cons]
				scalar `fyalt'= normalden(invnormal(`q'))
				if `fy'<=0 | `fy'>`fyalt' scalar `fy'=`fyalt'
				drop `ky'
				scalar `h`Zval'`Dval''=`hmean_`Zval'`Dval''*(`q'*(1-`q')/(`fy')^2)^(1/5)
				
				display "quantile: " `cent' " bw: " `h`Zval'`Dval''
				*set bw in Y direction
				if `h_half_`Zval'`Dval''<1 {
					scalar `h2_`Zval'`Dval''=max(`h_half_`Zval'`Dval''^5/`h`Zval'`Dval''^3,`h`Zval'`Dval''/10)
				}
				else {
					scalar `h2_`Zval'`Dval''=`h_half_`Zval'`Dval''^4/`h`Zval'`Dval''^3
				}
				scalar `h2_`Zval'`Dval''=`h2_`Zval'`Dval''*`h2factor_`Zval'`Dval''		
				replace `phi_`Zval'`Dval''=.5*(abs(`r'/`h`Zval'`Dval'')<=1)/`h`Zval'`Dval''
	    		}
			else scalar `h`Zval'`Dval''=999
		}
	}
	
	*Invert marginal distributions of Y0 and Y1
	
	centile `y' if `d'==0 & `z'==0,centile(`cent')
	scalar `y0'=r(c_1)
	
	centile `y' if `d'==1 & `z'==1,centile(`cent')
	scalar `y1'=r(c_1)
	
	sum `y'
	scalar `lo'=r(min)-2*r(sd)
	scalar `hi'=r(max) + r(sd)*2

	
	mata params0=(st_numscalar("`dplus'"),st_numscalar("`dminus'"), st_numscalar("`h2_10'"), st_numscalar("`h2_00'"), st_numscalar("`q'"), st_numscalar("`i10'"))
	mata fc0inv( "`y' `r' `d' `z' `phi_10' `phi_00'" , params0, st_numscalar("`y0'"), st_numscalar("`lo'"), st_numscalar("`hi'"))	
	
	scalar `FC0inv'=r(inv)
	scalar `conv0'=r(conv0)
	display "FC0inv, conv0: " `FC0inv' "," `conv0'
	
	mata params1=(st_numscalar("`dplus'"),st_numscalar("`dminus'"), st_numscalar("`h2_11'"), st_numscalar("`h2_01'"), st_numscalar("`q'"), st_numscalar("`i01'"))
	
	mata fc1inv( "`y' `r' `d' `z' `phi_11' `phi_01'", params1, st_numscalar("`y1'"), st_numscalar("`lo'"), st_numscalar("`hi'"))
	scalar `FC1inv'=r(inv)
	scalar `conv1'=r(conv1)
	display "FC1inv, conv1: " `FC1inv' "," `conv1'
	scalar `LQTE'=`FC1inv'-`FC0inv'
	matrix `lqte'[`qelem',1]=`LQTE'	
	}
 *Calculate complier densities
display "Now doing calcs for SEs"
foreach Zval in 1 0 {
    	foreach Dval in 1 0 {
			
			sum `y' if `z'==`Zval' & `d' ==`Dval'
			if `i`Zval'`Dval''==1 {
				scalar `hy'=1.06*r(sd)*r(N)^(-1/5)
				*Now do local linear density estimation:
				gen `ky' =1/`hy'*normalden((`y'-`FC`Dval'inv')/`hy')
				
				reg `ky' `r' [aw=`phimean_`Zval'`Dval''] if `z'==`Zval' & `d' ==`Dval'
				scalar `f`Zval'`Dval''=_b[_cons]
				drop `ky'
				
			}
			else scalar `f`Zval'`Dval''=0
			display "f`Zval'`Dval' is "
			display `f`Zval'`Dval''
		} /* end Dval */		
    } /* end Zval */
	
	scalar `f0'=abs((`f10'*(1-`dplus')-`f00'*(1-`dminus'))/(`dminus'-`dplus'))
	scalar `f1'=abs((`f11'*`dplus'-`f01'*`dminus')/(`dplus'-`dminus'))
* Calculate density of running variable
	
	sum `r'
	scalar `rbw'=r(N)^(-1/5)*r(sd)*1.06
	replace `kr'=1/`rbw'*normalden(`r'/`rbw')
	sum `kr'
	scalar `fr'=r(mean)
 *Calculate analytical variance
	
	scalar gp1=r(N)^(.2)*`h11'
	scalar gm1=r(N)^(.2)*`h01'
	scalar gp0=r(N)^(.2)*`h10'
	scalar gm0=r(N)^(.2)*`h00'
	scalar Ck=4
	scalar Dp1q=FD1Z1-`q'
	scalar Dp0q=FD0Z1-`q'
	scalar Dm1q=FD1Z0-`q'
	scalar Dm0q=FD0Z0-`q'
	scalar sigp1=FD1Z1*(1-FD1Z1)
	scalar sigp0=FD0Z1*(1-FD0Z1)
	scalar sigm1=FD1Z0*(1-FD1Z0)
	scalar sigm0=FD0Z0*(1-FD0Z0)
	scalar wp1qq=1/gp1*(sigp1*`dplus'+Dp1q^2*`dplus'*(1-`dplus'))
	scalar wm1qq=1/gm1*(sigm1*`dminus'+Dm1q^2*`dminus'*(1-`dminus'))
	scalar wp0qq=1/gp0*(sigp0*(1-`dplus')+Dp0q^2*`dplus'*(1-`dplus'))
	scalar wm0qq=1/gm0*(sigm0*(1-`dminus')+Dm0q^2*`dminus'*(1-`dminus'))
	scalar etamqq=1/sqrt(gm1*gm0)*(Dm0q*Dm1q*`dminus'*(1-`dminus'))
	scalar etapqq=1/sqrt(gp1*gp0)*(Dp0q*Dp1q*`dplus'*(1-`dplus'))
	scalar Cqq=(etapqq+etamqq)/`f1'/`f0'
	scalar V0qq=wp0qq+wm0qq/`f1'^2
	scalar V1qq=wp1qq+wm1qq/`f1'^2
	scalar vq=Ck/`fr'/(`dplus'-`dminus')^2*(V1qq+V0qq-2*Cqq)/r(N)^(4/5)
	matrix `se'[`qelem',1]=sqrt(vq)
	
	
}

}
restore
global nobs = _N
local nobs = _N
matrix `lqtec'=`lqte''
ereturn post `lqtec', esample(`touse')  depname(`y') obs($nobs)


ereturn local depvar "`y'"
ereturn local treatvar "`d'"
ereturn local runningvar "`running'"


ereturn matrix lqte `lqte'
ereturn matrix lb `lb'
ereturn matrix ub `ub' 
ereturn matrix se `se'
ereturn local cmd "_rdqteu" 
ereturn scalar conv0 =`conv0'
ereturn scalar conv1 =`conv1'

end

*Define MATA programs:
mata:
void fc0inv(string scalar varlist, real rowvector params, real scalar y0, real scalar lo, real scalar hi)
{
	rc0=mm_root(inv=y0, &myeval0(),lo,hi,0,1000,varlist,params)
	st_numscalar("r(conv0)",rc0)
	st_numscalar("r(inv)",inv)
}
end
mata:
function myeval0(y,string scalar varlist, real rowvector params)
	{
		q = params[5]
		// sprintf("current guess is: %f",y)
		diff=fc0(y, varlist, params)-q
		// sprintf("current obj is: %f",diff)
		return(diff)	
	}

end
mata:
function myeval1(y,string scalar varlist, real rowvector params)
	{
		q = params[5]
		diff=fc1(y, varlist, params)-q
		return(diff)
		
	}

end
mata:
void fc1inv(string scalar varlist,real rowvector params,real scalar y0, real scalar lo, real scalar hi)
{
	rc1=mm_root(inv=y0, &myeval1(),lo,hi,0,1000,varlist,params)
	st_numscalar("r(conv1)",rc1)
	st_numscalar("r(inv)",inv)
}
end

mata:
function fc0(real scalar y,string scalar varlist,real params )
{
	vars=tokens(varlist)
	Y=vars[1]
	R=vars[2]
	D=vars[3]
	Z=vars[4]
	phi_00=vars[6]
	h2_00=params[4]
	stata("gen D0Z0 = "+D+"==0 & "+Z+"==0")
	FD0Z0=Fdirect(y,Y,R,phi_00,h2_00,"D0Z0")
	// sprintf("FD0Z0: %f",FD0Z0)
	st_numscalar("FD0Z0",FD0Z0)
	i10=params[6]
	sprintf("i10: %f",i10)
	if (i10==0) {
		F=FD0Z0
		st_numscalar("FD0Z1",0)
	}
	else { 
		phi_10=vars[5]
		dplus=params[1]
		dminus=params[2]
		h2_10=params[3]
		// sprintf("dplus, dminus, h2_10: %f\t\t%f\t\t%f",dplus,dminus,h2_10)
		stata("gen D0Z1 = "+D+"==0 & "+Z+"==1")
		stata("sum "+phi_10+" if D0Z1")
		FD0Z1=Fdirect(y,Y,R,phi_10,h2_10,"D0Z1")
	//	sprintf("FD0Z1: %f",FD0Z1)
		st_numscalar("FD0Z1",FD0Z1)
		F=(FD0Z1*(1-dplus)-FD0Z0*(1-dminus))/(dminus-dplus)
		stata("drop D0Z1")
	}	
	stata("drop D0Z0")	
	return(F)
}
end

mata:
function fc1(real scalar y,string scalar varlist,real rowvector params)
{
	vars=tokens(varlist)
	Y=vars[1]
	R=vars[2]
	D=vars[3]
	Z=vars[4]
	phi_11=vars[5]
	h2_11=params[3]
	stata("gen D1Z1 = "+D+"==1 & "+Z+" ==1")
	FD1Z1=Fdirect(y,Y,R,phi_11,h2_11, "D1Z1")
	st_numscalar("FD1Z1",FD1Z1)
	i01=params[6]
	if (i01==0) {
		F=FD1Z1
		st_numscalar("FD1Z0",0)
	}
	else {
		phi_01=vars[6]
		dplus=params[1]
		dminus=params[2]	
		h2_01=params[4]
		stata("gen D1Z0 = "+D+"==1 & "+Z+"==0")
		FD1Z0=Fdirect(y,Y,R,phi_01,h2_01,"D1Z0")
		st_numscalar("FD1Z0",FD1Z1)
		F=(FD1Z1*dplus-FD1Z0*dminus)/(dplus-dminus)
		stata("drop D1Z0")
	}	
	stata("drop D1Z1")	
	return(F)
}
end

mata:
function Fdirect(real scalar y, string scalar Y,string scalar R,string scalar phi,real scalar h2, string scalar touse)
{
	st_view(Yvec,.,Y,touse)
	st_view(phivec,.,phi,touse)
	st_view(Rvec,.,R,touse)
	Sn1=colsum(phivec:*(-Rvec))
	Sn2=colsum(phivec:*(-Rvec):^2)
	w=phivec:*(Sn2:-(-Rvec):*Sn1)
	// sprintf("colsum(w): %f",colsum(w))
	F= 1/colsum(w):*colsum(w:*normal((y:-Yvec):/h2))
	
	return(F)
}
end

